//+------------------------------------------------------------------+
//|                                                      trading.mqh |
//|                                                Alexander Fedosov |
//|                                                                  |
//+------------------------------------------------------------------+
#property version   "1.08"
#property strict
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CTrading
  {
private:
   bool              useECNNDD;
   int               mgc;
   int               slpg;
   double            op_lt;
   bool              op_lt_of_bln;
   double            bln_risk;
   int               num_retr;
   color             b_clr;
   color             s_clr;

   void              isECN();
   bool              IsConnect();
   double            NL(double lot);
   bool              FreeM(double lot);
   bool              StopLev(double pr1,double pr2);
   bool              Freez(double pr1,double pr2);
   bool              CheckStop(int tip,double oop,double osl,double otp,double op,double sl,double tp,int mod=0);
   string            AddLeadingZero(string s,int k);

public:
   void              Trading(int magic_number=1111,int slippage=5,double open_lot=0.01,bool lot_of_balance=false,double risk=2.0,int number_of_try=5,
                             color buy_color=clrGreen,
                             color sell_color=clrTomato);

   double            Lots(int type);
   double            ND(double pr,int dg=0);
   int               NE(int pr);
   int               Dig();
   bool              OpnOrd(int tip,double lot,int take,int stop);
   int               SendOrd(int tip,double lot,double op,double sl,double tp);
   bool              ClosePosAll(int magik,int OrdType=-1);
   bool              isOpened(int m);
   void              TrailingStairs(int TrailingStart=30,int TrailingStop=25,int TrailStep=1);
   int               OrdExist(int direction);
   double            PriceBE(int dir);
   string            exp_name;
   int               typelot;
   bool              isTradeTime(int hour_begin=0,int minutes_begin=0,int hour_end=0,int minutes_end=0);
  };
//+------------------------------------------------------------------+
//|                                                       |
//+------------------------------------------------------------------+
void CTrading:: Trading(int magic_number,
                        int slippage,
                        double open_lot,
                        bool lot_of_balance,
                        double risk,
                        int number_of_try,
                        color buy_color,
                        color sell_color)
  {
   mgc=magic_number;
   slpg=slippage;
   op_lt=open_lot;
   op_lt_of_bln=lot_of_balance;
   bln_risk=risk;
   num_retr=number_of_try;
   b_clr=buy_color;
   s_clr=sell_color;
   isECN();
  }
//+------------------------------------------------------------------+
//|                                       |
//+------------------------------------------------------------------+
int CTrading:: Dig()
  {
   return((_Digits==5 || _Digits==3 || _Digits==1)?10:1);
  }
//+------------------------------------------------------------------+
//|                                              |
//+------------------------------------------------------------------+
void CTrading:: isECN()
  {
   switch(int(SymbolInfoInteger(_Symbol,SYMBOL_TRADE_EXEMODE)))
     {
      case SYMBOL_TRADE_EXECUTION_MARKET:
         useECNNDD=true;
         slpg=0;
         break;
      default:
         useECNNDD=false;
         break;
     }
  }
//+------------------------------------------------------------------+
//|                            |
//+------------------------------------------------------------------+
bool CTrading:: IsConnect()
  {
   return((!TerminalInfoInteger(TERMINAL_CONNECTED) || !TerminalInfoInteger(TERMINAL_TRADE_ALLOWED) || IsTradeContextBusy())?false:true);
  }
//+------------------------------------------------------------------+
//|                                          |
//| 1 -  , 2 -  , 3 -                        |
//+------------------------------------------------------------------+
double CTrading:: Lots(int type=1)
  {
   type=(type>3 || type<1)?1:type;
   double res,bal;
   switch(type)
     {
      case 1: bal = AccountBalance();break;
      case 2: bal = AccountEquity();break;
      case 3: bal = AccountFreeMargin();break;
      default:bal = AccountBalance();break;
     }
   if(!op_lt_of_bln)
      res=op_lt;
   else
   if(bln_risk>0.0 && op_lt_of_bln)
      res=NL((bal/(100.0/bln_risk))/MarketInfo(_Symbol,MODE_MARGINREQUIRED));
   return(res);
  }
//+------------------------------------------------------------------+
//|                                                  |
//+------------------------------------------------------------------+
double CTrading:: ND(double pr,int dg=0)
  {
   if(dg==0)
      dg=_Digits;
   return(NormalizeDouble(pr,dg));
  }
//+------------------------------------------------------------------+
//|                                         |
//+------------------------------------------------------------------+
double CTrading:: NL(double lot)
  {
   int mf=int(MathCeil(lot/SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_STEP)));
   double res=mf*SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_STEP);
   res=MathMax(res,SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_MIN));
   res=MathMin(res,SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_MAX));
   return(res);
  }
//+------------------------------------------------------------------+
//|                                           |
//+------------------------------------------------------------------+
int CTrading:: NE(int pr)
  {
   long res=SymbolInfoInteger(_Symbol,SYMBOL_TRADE_STOPS_LEVEL);
   res++;
   if(pr>res)
      res=pr;
   return(int(res));
  }
//+------------------------------------------------------------------+
//|                                             |
//+------------------------------------------------------------------+
bool CTrading:: OpnOrd(int tip,double lot,int take,int stop)
  {
   bool res=false;
   double sl=0.0,tp=0.0,opr=0.0;
   switch(tip)
     {
      case 0:
         opr=Ask;
         break;
      case 1:
         opr=Bid;
         break;
     }
   if(MathMod(tip,2.0)==0.0)
     {
      if(stop>0)
         sl=opr-NE(stop*Dig())*_Point;
      if(take>0)
         tp=opr+NE(take*Dig())*_Point;
     }
   else
     {
      if(stop>0)
         sl=opr+NE(stop*Dig())*_Point;
      if(take>0)
         tp=opr-NE(take*Dig())*_Point;
     }
   if(SendOrd(tip,Lots(typelot),opr,sl,tp)>0)
      res=true;
   else
      GetLastError();
   return(res);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int CTrading:: SendOrd(int tip,double lot,double op,double sl,double tp)
  {
   int i=0,tiket=0;
   if(!FreeM(lot))
     {
      Print(exp_name,": Not enough money for trading!");
      return(tiket);
     }
   color col=s_clr;
   if(MathMod(tip,2.0)==0.0)
      col=b_clr;
   for(i=0;i<num_retr;i++)
     {
      if(!IsConnect())
         Sleep(4000);
      RefreshRates();
      switch(tip)
        {
         case 0:
            op=Ask;
            break;
         case 1:
            op=Bid;
            break;
        }
      if(!CheckStop(tip,0.0,0.0,0.0,op,sl,tp,0))
         break;
      tiket=OrderSend(_Symbol,tip,lot,ND(op),slpg,ND(sl),ND(tp),exp_name,mgc,0,col);
      if(tiket>0)
         break;
     }
   return(tiket);
  }
//+------------------------------------------------------------------+
//|                         |
//| oop,osl,otp - ,                                   |
//|                                                                  |
//+------------------------------------------------------------------+
bool CTrading:: CheckStop(int tip,double oop,double osl,double otp,double op,double sl,double tp,int mod=0)
  {
   bool res=true;
   double pro=0.0,prc=0.0;
   if(MathMod(tip,2.0)==0.0)
     {pro=Ask;prc=Bid;}
   else
     {pro=Bid;prc=Ask;}
   switch(mod)
     {
      case 0: //  .
         switch(tip)
           {
            case 0:
            if(sl>0.0 && !StopLev(prc,sl)){res=false;break;}
            if(tp>0.0 && !StopLev(tp,prc)){res=false;break;}
            break;
            case 1:
            if(sl>0.0 && !StopLev(sl,prc)){res=false;break;}
            if(tp>0.0 && !StopLev(prc,tp)){res=false;break;}
            break;
            case 2:
            if(!StopLev(pro,op)){res=false;break;}
            if(sl>0.0 && !StopLev(op,sl)){res=false;break;}
            if(tp>0.0 && !StopLev(tp,op)){res=false;break;}
            break;
            case 3:
            if(!StopLev(op,pro)){res=false;break;}
            if(sl>0.0 && !StopLev(sl,op)){res=false;break;}
            if(tp>0.0 && !StopLev(op,tp)){res=false;break;}
            break;
            case 4:
            if(!StopLev(op,pro)){res=false;break;}
            if(sl>0.0 && !StopLev(op,sl)){res=false;break;}
            if(tp>0.0 && !StopLev(tp,op)){res=false;break;}
            break;
            case 5:
            if(!StopLev(pro,op)){res=false;break;}
            if(sl>0.0 && !StopLev(sl,op)){res=false;break;}
            if(tp>0.0 && !StopLev(op,tp)){res=false;break;}
            break;
           }
         break;
      case 1: //  .
         switch(tip)
           {
            case 0:
            if(osl>0.0 && !Freez(prc,osl)){res=false;break;}
            if(otp>0.0 && !Freez(otp,prc)){res=false;break;}
            break;
            case 1:
            if(osl>0.0 && !Freez(osl,prc)){res=false;break;}
            if(otp>0.0 && !Freez(prc,otp)){res=false;break;}
            break;
           }
         break;
      case 2: //  .
         if(prc>oop)
           {
            if(!Freez(prc,oop))
              {res=false;break;}
           }
         else
           {
            if(!Freez(oop,prc))
              {res=false;break;}
           }
         break;
      case 3: // .
         switch(tip)
           {
            case 0:
            if(osl>0.0 && !Freez(prc,osl)){res=false;break;}
            if(otp>0.0 && !Freez(otp,prc)){res=false;break;}
            if(sl>0.0 && !StopLev(prc,sl)){res=false;break;}
            if(tp>0.0 && !StopLev(tp,prc)){res=false;break;}
            break;
            case 1:
            if(osl>0.0 && !Freez(osl,prc)){res=false;break;}
            if(otp>0.0 && !Freez(prc,otp)){res=false;break;}
            if(sl>0.0 && !StopLev(sl,prc)){res=false;break;}
            if(tp>0.0 && !StopLev(prc,tp)){res=false;break;}
            break;
            case 2:
            if(sl>0.0 && !StopLev(op,sl)){res=false;break;}
            if(tp>0.0 && !StopLev(tp,op)){res=false;break;}
            if(!StopLev(pro,op) || !Freez(pro,op)){res=false;break;}
            break;
            case 3:
            if(sl>0.0 && !StopLev(sl,op)){res=false;break;}
            if(tp>0.0 && !StopLev(op,tp)){res=false;break;}
            if(!StopLev(op,pro) || !Freez(op,pro)){res=false;break;}
            break;
            case 4:
            if(sl>0.0 && !StopLev(op,sl)){res=false;break;}
            if(tp>0.0 && !StopLev(tp,op)){res=false;break;}
            if(!StopLev(op,pro) || !Freez(op,pro)){res=false;break;}
            break;
            case 5:
            if(sl>0.0 && !StopLev(sl,op)){res=false;break;}
            if(tp>0.0 && !StopLev(op,tp)){res=false;break;}
            if(!StopLev(pro,op) || !Freez(pro,op)){res=false;break;}
            break;
           }
         break;
     }
   return(res);
  }
//+------------------------------------------------------------------+
//|                                  |
//+------------------------------------------------------------------+
bool CTrading::ClosePosAll(int magik,int OrdType=-1)
  {
   double price;
   int i;
   bool _Ans=true;
   for(int pos=OrdersTotal()-1; pos>=0; pos--)
     {
      if(!OrderSelect(pos,SELECT_BY_POS,MODE_TRADES))
         continue;
      if(OrderSymbol()!=_Symbol || OrderMagicNumber()!=magik)
         continue;
      int order_type=OrderType();
      if(order_type>1 || (OrdType>=0 && OrdType!=order_type))
         continue;
      RefreshRates();
      i=0;
      bool Ans=false;
      while(!Ans && i<num_retr)
        {
         price=(order_type==OP_BUY)?Bid:Ask;
         Ans=OrderClose(OrderTicket(),OrderLots(),ND(price),slpg);
         i++;
        }
      if(!Ans)
         _Ans=false;
     }
   return(_Ans);
  }
//+------------------------------------------------------------------+
//|                                             |
//+------------------------------------------------------------------+
bool CTrading::isOpened(int m)
  {
   int i=0;
   for(int pos=OrdersTotal()-1; pos>=0; pos--)
     {
      if(!OrderSelect(pos,SELECT_BY_POS,MODE_TRADES))
         continue;
      if(OrderSymbol()==_Symbol && OrderMagicNumber()==m)
         i++;
     }
   return((i>0)?true:false);
  }
//+------------------------------------------------------------------+
//|   -                           |
//+------------------------------------------------------------------+
bool CTrading:: FreeM(double lot)
  {
   double a,b;
   a=lot*MarketInfo(_Symbol,MODE_MARGINREQUIRED);
   b= AccountFreeMargin();
   return (a>b)?false:true;
  }
//+------------------------------------------------------------------+
//|   -                              |
//+------------------------------------------------------------------+
bool CTrading:: StopLev(double pr1,double pr2)
  {
   return(long(MathCeil((pr1-pr2)/Point))<=SymbolInfoInteger(_Symbol,SYMBOL_TRADE_STOPS_LEVEL))?false:true;
  }
//+------------------------------------------------------------------+
//|                    |
//+------------------------------------------------------------------+
bool CTrading:: Freez(double pr1,double pr2)
  {
   return(long(MathCeil((pr1-pr2)/_Point))<=SymbolInfoInteger(_Symbol,SYMBOL_TRADE_FREEZE_LEVEL))?false:true;
  }
//+------------------------------------------------------------------+
//|                   |
//+------------------------------------------------------------------+
int CTrading:: OrdExist(int direction)
  {
   int OrdCount=0;
   if(OrdersTotal()>0)
     {
      for(int i=0; i<=OrdersTotal(); i++)
        {
         if(!OrderSelect(i,SELECT_BY_POS,MODE_TRADES))
            continue;
         if(OrderMagicNumber()!=mgc || OrderSymbol()!=Symbol())
            continue;
         if(OrderType()==direction)
            OrdCount++;
        }
     }
   return (OrdCount);
  }
//+------------------------------------------------------------------+
//|                                               |
//+------------------------------------------------------------------+
//     TrailingStart     TrailingStop     TrailStep
void CTrading:: TrailingStairs(int TrailingStart=30,int TrailingStop=25,int TrailStep=1)
  {
   double PriceBEBuy=PriceBE(OP_BUY);
   double PriceBESell=PriceBE(OP_SELL);
   int BuyOrders=OrdExist(OP_BUY);
   int SellOrders=OrdExist(OP_SELL);
   if(BuyOrders>0 || SellOrders>0)
     {
      TrailingStart= TrailingStart*Dig();
      TrailingStop = TrailingStop*Dig();
      TrailStep= TrailStep*Dig();
      for(int i=0; i<=OrdersTotal(); i++)
        {
         if(!OrderSelect(i,SELECT_BY_POS,MODE_TRADES))
            break;
         if(OrderMagicNumber()!=mgc || OrderSymbol()!=Symbol())
            continue;
         if(OrderType()==OP_BUY)
           {
            if((Bid-TrailingStart*_Point>PriceBEBuy) && ((OrderStopLoss()!=0 && Bid-TrailingStop*_Point>OrderStopLoss()+TrailStep*_Point) || OrderStopLoss()==0))
              {
               if(!OrderModify(OrderTicket(),OrderOpenPrice(),NormalizeDouble(Bid-TrailingStop*_Point,Digits()),OrderTakeProfit(),0,Green))
                  Print(exp_name,": Error standard trailing stop! - ",GetLastError());
              }
           }
         if(OrderType()==OP_SELL)
           {
            if((Ask+TrailingStart*_Point<PriceBESell) && ((OrderStopLoss()!=0 && Ask+TrailingStop*_Point<OrderStopLoss()-TrailStep*_Point) || OrderStopLoss()==0))
              {
               if(!OrderModify(OrderTicket(),OrderOpenPrice(),NormalizeDouble(Ask+TrailingStop*_Point,Digits()),OrderTakeProfit(),0,Green))
                  Print(exp_name,": Error standard trailing stop! - ",GetLastError());
              }
           }
        }
     }
   return;
  }
//+------------------------------------------------------------------+
//|                     |
//|                                              |
//+------------------------------------------------------------------+
double CTrading:: PriceBE(int dir)
  {
   double BEPrice=0;
   double MyBEPrice=0;
   double BuyLots=0;
   double SellLots=0;
   double BuyProfit=0;
   double SellProfit=0;
   double BuyLevel=0;
   double SellLevel=0;
   if(OrdExist(OP_BUY)>0 || OrdExist(OP_SELL)>0)
     {
      for(int i=OrdersTotal(); i>=0; i--)
        {
         if(!OrderSelect(i,SELECT_BY_POS,MODE_TRADES)) continue;
         if(OrderMagicNumber()!=mgc || OrderSymbol()!=Symbol()) continue;
         if(OrderType()==OP_BUY)
           {
            BuyLots=BuyLots+OrderLots();
            BuyProfit=BuyProfit+OrderProfit()+OrderCommission()+OrderSwap();
           }
         if(OrderType()==OP_SELL)
           {
            SellLots=SellLots+OrderLots();
            SellProfit=SellProfit+OrderProfit()+OrderCommission()+OrderSwap();
           }
        }
     }
   double TickValue=MarketInfo(Symbol(),MODE_TICKVALUE);
   if(BuyLots>0) BuyLevel=NormalizeDouble(Bid-(BuyProfit/(TickValue*BuyLots)*Point),Digits); else BuyLevel=0;
   if(SellLots>0) SellLevel=NormalizeDouble(Ask+(SellProfit/(TickValue*SellLots)*Point),Digits); else SellLevel=0;
   if((BuyLots-SellLots)>0) BEPrice=NormalizeDouble(Bid-((BuyProfit+SellProfit)/(TickValue*(BuyLots-SellLots))*Point),Digits);
   if((SellLots-BuyLots)>0) BEPrice=NormalizeDouble(Ask+((BuyProfit+SellProfit)/(TickValue*(SellLots-BuyLots))*Point),Digits);
   if(dir==OP_BUY)   MyBEPrice=BuyLevel;
   if(dir==OP_SELL)  MyBEPrice=SellLevel;
   if(dir==-1)       MyBEPrice=BEPrice;
   return(NormalizeDouble(MyBEPrice,Digits));
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CTrading:: isTradeTime(int hour_begin=0,int minutes_begin=0,int hour_end=0,int minutes_end=0)
  {
   datetime db, de;           //     
   int      hc;               //     
   string hour_begin_txt=AddLeadingZero(IntegerToString(hour_begin),2);
   string minutes_begin_txt=AddLeadingZero(IntegerToString(minutes_begin),2);
   string hour_end_txt=AddLeadingZero(IntegerToString(hour_end),2);
   string minutes_end_txt=AddLeadingZero(IntegerToString(minutes_end),2);
   db=StrToTime(TimeToStr(TimeCurrent(), TIME_DATE)+" "+hour_begin_txt+":"+minutes_begin_txt);
   de=StrToTime(TimeToStr(TimeCurrent(), TIME_DATE)+" "+hour_end_txt+":"+minutes_end_txt);
   hc=TimeHour(TimeCurrent());
   if(db>=de)
      if(hc>=hour_end)
         de+=24*60*60;
   else
      db-=24*60*60;

   if(TimeCurrent()>=db && TimeCurrent()<=de)
      return(true);
   else
      return(false);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
string CTrading:: AddLeadingZero(string s,int k)
  {
   while(StringLen(s)<k)
      s=StringConcatenate("0",s);
   return(s);
  }
//+----------------------------------------------------------------------------+

//if(Reverse)
//  {
//   if(psar[2]>Close[2] && psar[1]<Close[1] && psar[0]<Close[0] && !tr.isOpened(magic))
//      tr.OpnOrd(OP_BUY,lt,0,0);
//   if(psar[2]<Close[2] && psar[1]>Close[1] && psar[0]>Close[0] && !tr.isOpened(magic))
//      tr.OpnOrd(OP_SELL,lt,0,0);
//   if(psar[2]>Close[2] && psar[1]<Close[1] && psar[0]<Close[0] && tr.isOpened(magic))
//      tr.ClosePosAll(magic,OP_SELL);
//   if(psar[2]<Close[2] && psar[1]>Close[1] && psar[0]>Close[0] && tr.isOpened(magic))
//      tr.ClosePosAll(magic,OP_BUY);
//  }
